/***********************************************************************

GECCO, a platform for command and control gaming

Copyright (C) 2001 Henrik Brnhielm, Andreas Enblom, Niklas
Hallenfur, Karl Hasselstrm, Henrik Hgerstrm, Oskar Linde, Jing Fu
Zi and Jon slund

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or (at
your option) any later version.

This program is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
USA.

***********************************************************************/

package gecco.communication.messages;

import java.io.*;

/**
 *  This class is put in the send queue, on the clients side of the
 *  communication, when the client calls abortAction()
 *
 * @author     Niklas Hallenfur
 * @author     Karl Hasselstrm
 * @created    June 27, 2001
 */

public class AbortActionMessage extends ActionMessage {

    /**
     *  Constructor for the AbortActionMessage object
     *
     * @param  actionId  Description of Parameter
     */
    public AbortActionMessage(long actionId) {
        super(actionId, 0, "");
    }


    /**
     *  This method writes the message to the given
     *  <code>DataOutput</code>.
     *
     * @param out The output stream.
     * @exception  IOException  In case something goes wrong.
     */
    public void write(DataOutput out) throws IOException {

	out.writeInt(MessageType.AbortAction);
	
	out.writeLong(getId());
    }


    /**
     *  This factory method reads a message of this type without an
     *  initial ID from the input stream, and returns a
     *  <code>Message</code> object.
     *
     * @param  in  The input stream.
     * @return     An instance of <code>Message</code>.
     */
    public static Message read(DataInput in) throws IOException {

	long actionId = in.readLong();

	return new AbortActionMessage(actionId);
    }

}
