/***********************************************************************

GECCO, a platform for command and control gaming

Copyright (C) 2001 Henrik Brnhielm, Andreas Enblom, Niklas
Hallenfur, Karl Hasselstrm, Henrik Hgerstrm, Oskar Linde, Jing Fu
Zi and Jon slund

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or (at
your option) any later version.

This program is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
USA.

***********************************************************************/

package gecco.communication.messages;

import java.io.*;
import java.util.*;

/**
 *  This is a list of the currently available roles.
 *
 * @author     Karl Hasselstrm
 * @created    July 26, 2001
 */
public class AvailableRolesMessage extends Message {

    public Set roles;


    public AvailableRolesMessage(Set roles) {
	this.roles = roles;
    }


    /**
     *  This method writes the message to the given
     *  <code>DataOutput</code>.
     *
     * @param out The output stream.
     * @exception  IOException  In case something goes wrong.
     */
    public void write(DataOutput out) throws IOException {

	out.writeInt(MessageType.AvailableRoles);

	out.writeInt(roles.size());
	Iterator iter = roles.iterator();
	while (iter.hasNext()) {
	    out.writeUTF((String) iter.next());
	}
    }


    /**
     *  This factory method reads a message of this type without an
     *  initial ID from the input stream, and returns a
     *  <code>Message</code> object.
     *
     * @param  in  The input stream.
     * @return     An instance of <code>Message</code>.
     */
    public static Message read(DataInput in) throws IOException {

	int numRoles = in.readInt();
	Set roles = new HashSet(numRoles);
	for (int i = 0; i < numRoles; i++) {
	    roles.add(in.readUTF());
	}

	return new AvailableRolesMessage(roles);
    }

}
