/***********************************************************************

GECCO, a platform for command and control gaming

Copyright (C) 2001 Henrik Brnhielm, Andreas Enblom, Niklas
Hallenfur, Karl Hasselstrm, Henrik Hgerstrm, Oskar Linde, Jing Fu
Zi and Jon slund

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or (at
your option) any later version.

This program is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
USA.

***********************************************************************/

package gecco.communication.messages;

import java.awt.*;
import java.io.*;
import java.util.*;

/**
 *  This message contains the map and the icons.
 *
 * @author     Karl Hasselstrm
 * @created    July 26, 2001
 */
public class MapMessage extends Message {

    public Dimension size;
    public byte[] mapRGB;
    public Map icons;


    public MapMessage(Dimension size, byte[] mapRGB, Map icons) {
	this.size = size;
	this.mapRGB = mapRGB;
	this.icons = icons;
    }


    /**
     *  This method writes the message to the given
     *  <code>DataOutput</code>.
     *
     * @param out The output stream.
     * @exception  IOException  In case something goes wrong.
     */
    public void write(DataOutput out) throws IOException {

	out.writeInt(MessageType.Map);
	
	// Write the map.
	out.writeInt(size.width);
	out.writeInt(size.height);
	out.write(mapRGB);

	// Write the icons.
	Iterator types = icons.keySet().iterator();
	out.writeInt(icons.size());
	while (types.hasNext()) {
	    String type = (String) types.next();
	    byte[] icon = (byte[]) icons.get(type);
	    out.writeUTF(type);
	    out.writeInt(icon.length);
	    out.write(icon);
	}
    }


    /**
     *  This factory method reads a message of this type without an
     *  initial ID from the input stream, and returns a
     *  <code>Message</code> object.
     *
     * @param  in  The input stream.
     * @return     An instance of <code>Message</code>.
     */
    public static Message read(DataInput in) throws IOException {

	// The size of the map.
	int width = in.readInt();
	int height = in.readInt();
	Dimension size = new Dimension(width, height);
	
	// The map.
	int mapArraySize = width * height * 3;
	byte[] mapRGB = new byte[mapArraySize];
	for (int i = 0; i < height; i++) {
	    in.readFully(mapRGB, width * i * 3, 3 * width);
	}
	
	// Get number of types.
	int numUnitTypes = in.readInt();
	Map icons = new HashMap(numUnitTypes);
	
	// Read the .gif/.jpg-image icons for the unit types.
	for (int i = 0; i < numUnitTypes; i++) {
	    String type = in.readUTF();
	    int numBytes = in.readInt();
	    byte[] icon = new byte[numBytes];
	    in.readFully(icon);
	    icons.put(type, icon);
	}

	// Return the message.
	return new MapMessage(size, mapRGB, icons);
    }
    
}
