/***********************************************************************

GECCO, a platform for command and control gaming

Copyright (C) 2001 Henrik Brnhielm, Andreas Enblom, Niklas
Hallenfur, Karl Hasselstrm, Henrik Hgerstrm, Oskar Linde, Jing Fu
Zi and Jon slund

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or (at
your option) any later version.

This program is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
USA.

***********************************************************************/

package gecco.communication.server;

import gecco.client.Action;
import gecco.communication.messages.*;
import gecco.communication.server.*;
import gecco.server.core.ActionDefinition;
import gecco.server.core.Server;
import gecco.server.core.ReferenceHolder;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.image.BufferedImage;
import java.io.*;
import java.util.*;

/**
 *  a class that listens for incoming messages from the client, receives them,
 *  and tells the server about them
 *
 * @author     Niklas Hallenfur
 * @author     Karl Hasselstrm
 * @author     Henrik Hgerstrm
 * @author     Jon slund
 * @created    June 27, 2001
 */
public class DataOutputExchangeHandler implements Runnable {
    /**
     *  Description of the Field
     */
    public Boolean pingSyncObject;

    /**
     *  Description of the Field
     */
    private MessageQueue messages_to_send;
    /**
     *  Description of the Field
     */
    private ClientConnection connection;
    /**
     *  Description of the Field
     */
    private Server server;
    /**
     *  Description of the Field
     */
    private ClientConnectionHandler i_am_working_for;
    /**
     *  Description of the Field
     */
    private boolean disconnect;


    /**
     *  Constructor for the DataOutputExchangeHandler object
     *
     * @param  q    Description of Parameter
     * @param  cc   Description of Parameter
     * @param  se   Description of Parameter
     * @param  cch  Description of Parameter
     */
    public DataOutputExchangeHandler(MessageQueue q, ClientConnection cc,
            Server se, ClientConnectionHandler cch) {
        messages_to_send = q;
        connection = cc;
        server = se;
        i_am_working_for = cch;
        disconnect = false;
        // Just a dummy object to synchronize on, the class doesn't matter at all...
        pingSyncObject = new Boolean(true);
    }


    /**
     *  Adds a feature to the MessageToQueue attribute of the
     *  DataOutputExchangeHandler object
     *
     * @param  message  The feature to be added to the MessageToQueue attribute
     */
    public void addMessageToQueue(Message message) {
        synchronized (messages_to_send) {
            messages_to_send.add(message);
            messages_to_send.notifyAll();
        }
    }


    /**
     *  Main processing method for the DataOutputExchangeHandler object
     */
    public void run() {
        while (!disconnect) {
            try {
                Message m = null;
                // Wait until a message is placed in the queue if it's empty!
                synchronized (messages_to_send) {
                    if (messages_to_send.isEmpty()) {
                        messages_to_send.wait();
                    }
                    m = messages_to_send.get();
                }
                // Release sync on message queue, and send message!
                if (m != null) {
                    sendMessage(m);
                }
                // Yield to other threads!
                Thread.yield();
            } catch (Exception e) {

		ReferenceHolder.debug.println("Exception occurred while sending message to client:");
		e.printStackTrace(ReferenceHolder.debug);
		ReferenceHolder.debug.println("The client was disconnected, but the " +
					      "server is unharmed.");
                i_am_working_for.connectionLost(connection.getRole(),
                        connection.getId());
                break;
            }
        }
    }


    /**
     *  called when the client says it wants to abort the game or disconnect
     */
    public void abortGame() {
        i_am_working_for.connectionLost(connection.getRole(),
                connection.getId());
    }


    /**
     *  called whenever someone wants this Thread to die
     */
    public void kill() {
        disconnect = true;
    }


    /**
     *  Called when a message is to be sent.
     *
     * @param  m              Description of Parameter
     * @exception  Exception  Description of Exception
     */
    private void sendMessage(Message m) throws Exception {

        // Send and receive a ping, to ensure that the client is ready
        // to receive a message!
	//ping();

	// Send the message.
	m.write(connection.getDataOutputStream());

	// Make sure it's sent right away and not later.
        connection.getDataOutputStream().flush();
    }


    /**
     *  send a 'ping' to see if this client is available
     *
     * @exception  Exception  Description of Exception
     */
    private void ping() throws Exception {
        connection.getDataOutputStream().writeInt(MessageType.Ping);
        connection.getDataOutputStream().flush();
        synchronized (pingSyncObject) {
            pingSyncObject.wait();
        }
    }

}
