/***********************************************************************

GECCO, a platform for command and control gaming

Copyright (C) 2001 Henrik Brnhielm, Andreas Enblom, Niklas
Hallenfur, Karl Hasselstrm, Henrik Hgerstrm, Oskar Linde, Jing Fu
Zi and Jon slund

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or (at
your option) any later version.

This program is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
USA.

***********************************************************************/

package gecco.communication.server;

import java.awt.*;
import java.io.*;
import java.net.*;
import java.util.*;
import java.util.Map;

import gecco.communication.messages.*;
import gecco.server.core.*;

/**
 *  A class that will listen for incoming connections from clients, and handle
 *  the initial communication, until the client successfully joins as a role,
 *  and calls startGame. all communication with the client will thereafter be
 *  handled by a DataExchangeHandler
 *
 * @author     Niklas Hallenfur
 * @author     Jon slund
 * @created    June 27, 2001
 */

class NewClientListener implements Runnable {

    /**
     *  Description of the Field
     */
    private DataInputStream data_in;
    /**
     *  Description of the Field
     */
    private DataOutputStream data_out;

    /**
     *  Description of the Field
     */
    private ClientAmbassador started_by;
    /**
     *  Description of the Field
     */
    private ServerSocket server;


    /**
     * @param  port             The port where to listen for incoming
     *      connections
     * @param  s                Description of Parameter
     * @exception  IOException  Description of Exception
     */

    public NewClientListener(ClientAmbassador s, int port) throws IOException {
        started_by = s;
        try {
            server = new ServerSocket(port);
        } catch (IOException e) {
            ReferenceHolder.err.println("NewClientListener(): Could not create ServerSocket");
            throw e;
        }
    }


    public void run() {
        while (true) {
            String role = null;
            try {
                StringTokenizer st;
                Socket s = accept();
                boolean handshake = true;

                while (handshake) {

		    // Flush any old messages.
		    data_out.flush();

		    Message m = Message.read(data_in);
		    if(m instanceof DisconnectMessage) {
			handshake = false;
		    } else if(m instanceof GetAvailableRolesMessage) {
			Message reply = new AvailableRolesMessage(started_by.rolesLeft());
			reply.write(data_out);
		    } else if(m instanceof JoinAsRoleMessage) {
			role = ((JoinAsRoleMessage) m).role;
			connectToGame(s, role);
		    } else if(m instanceof GetMapMessage) {
			Message reply = new MapMessage(started_by.getMapSize(),
						       started_by.getMap(),
						       started_by.getUnitTypeIcons());
			reply.write(data_out);
		    } else if(m instanceof StartGameMessage) {
			startGame(s, role);
			handshake = false;
		    } else {
			throw new Exception("Wrong message!");
		    }
		    
		}
		
            } catch (Exception e) {
		// If the server should tell us about connection
		// startup problems, this is the place to do so.
            }
        }
    }
    


    /**
     *  Description of the Method
     *
     * @return    Description of the Returned Value
     */
    public Socket accept() {
        try {
            Socket s = server.accept();
	    s.setTcpNoDelay(true);

            data_in = new DataInputStream(s.getInputStream());
            data_out = new DataOutputStream(s.getOutputStream());

            return s;
        } catch (Exception e) {
            return null;
        }

    }


    /**
     *  Description of the Method
     *
     * @param  s     Description of Parameter
     * @param  role  Description of Parameter
     * @return       Description of the Returned Value
     */

    private boolean connectToGame(Socket s, String role) throws Exception {

	if (started_by.acceptRole(role)) {
	    Message m = new AcceptConnectionMessage();
	    m.write(data_out);
	    return true;
	} else {
	    Message m = new RejectConnectionMessage();
	    m.write(data_out);
	    return false;
	}
    }


    /**
     *  Description of the Method
     *
     * @param  s     Description of Parameter
     * @param  role  Description of Parameter
     */
    private void startGame(Socket s, String role) {
        try {
            int id = started_by.getAvailableId(role);

            /*
             * skapa en ny ClientConnection och lgg till
             */
            ClientConnection c = new ClientConnection(s, role, id);
            started_by.addClient(c);
        } catch (Exception e) {
        }
    }

}
