/***********************************************************************

GECCO, a platform for command and control gaming

Copyright (C) 2001 Henrik Brnhielm, Andreas Enblom, Niklas
Hallenfur, Karl Hasselstrm, Henrik Hgerstrm, Oskar Linde, Jing Fu
Zi and Jon slund

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or (at
your option) any later version.

This program is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
USA.

***********************************************************************/

package gecco.game.testgame;

import gecco.server.core.*;

/**
 *  Description of the Class
 *
 */
public class InstantAttackAction extends Action {

    /**
     *  Description of the Field
     */
    private double addX;
    /**
     *  Description of the Field
     */
    private double addY;
    /**
     *  Description of the Field
     */
    private double stepLength;
    /**
     *  Description of the Field
     */
    private int attackFactor;
    /**
     *  Description of the Field
     */
    private boolean isAttacking = false;

    /**
     *  Description of the Field
     */
    private double attackRadiusSquared;


    /**
     *  Constructor for the InstantAttackAction object
     */
    public InstantAttackAction() {
        super();
    }


    // Override superclass methods

    /**
     *  Description of the Method
     *
     * @param  _actionHandle  Description of Parameter
     * @param  _unitHandle    Description of Parameter
     */
    public void initiate(int _actionHandle, int _unitHandle) {
        throw new IllegalArgumentException("This action must be instanciated with a point as an argument.");
    }


    /**
     *  Description of the Method
     *
     * @param  _actionHandle   Description of Parameter
     * @param  _unitHandle     Description of Parameter
     * @param  _argUnitHandle  Description of Parameter
     */
    public void initiate(int _actionHandle, int _unitHandle, int _argUnitHandle) {
        super.initiate(_actionHandle, _unitHandle, _argUnitHandle);
        // Set the attack factor and stepLength depending on which unittype
        // that is attacking!
        if (getUnit().getUnitType().getName().endsWith("Helicopter")) {
            stepLength = 2.0;
            attackFactor = 2;
            attackRadiusSquared = 35 * 35;
        } else if (getUnit().getUnitType().getName().endsWith("Tank")) {
            stepLength = 0.9;
            attackFactor = 8;
            attackRadiusSquared = 25 * 25;
        } else {
            // Unknown, set default values
            stepLength = 1;
            attackFactor = 3;
        }
    }


    /**
     *  Description of the Method
     *
     * @param  _actionHandle  Description of Parameter
     * @param  _unitHandle    Description of Parameter
     * @param  _argX          Description of Parameter
     * @param  _argY          Description of Parameter
     */
    public void initiate(int _actionHandle, int _unitHandle, double _argX, double _argY) {
        throw new IllegalArgumentException("This action must be instanciated with a unit as an argument.");
    }


    /**
     *  Description of the Method
     *
     * @return    Description of the Returned Value
     */
    public ActionReturn checkPoint() {

        if (getArgumentUnit().isDestroyed()) {
            return new ActionReturnCompleted("Attack successful! Target destroyed!");
        }

        String unitMessage = null;

        double curX = getUnit().getX();
        double curY = getUnit().getY();
        double enemyX = getArgumentUnit().getX();
        double enemyY = getArgumentUnit().getY();

        // Check if we're inside the attack radius!
        double dx = enemyX - curX;
        double dy = enemyY - curY;
        double dist = dx * dx + dy * dy;

        // Check if unit is visible
        if (!canSeePosition(enemyX, enemyY)) {
            unitMessage = "Target lost, instant attack aborted.";
        } else if (dx * dx + dy * dy <= attackRadiusSquared) {
            // YES! ATTACK!
            sendEventToUnit(getArgumentUnit(), "ATTACK", 10);
            unitMessage = "Instant attack performed OK.";
        } else {
            unitMessage = "Target outside the attack radius. Instant attack aborted.";
        }

        return new ActionReturnCompleted(unitMessage);
    }
}
