/***********************************************************************

GECCO, a platform for command and control gaming

Copyright (C) 2001 Henrik Brnhielm, Andreas Enblom, Niklas
Hallenfur, Karl Hasselstrm, Henrik Hgerstrm, Oskar Linde, Jing Fu
Zi and Jon slund

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or (at
your option) any later version.

This program is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
USA.

***********************************************************************/

package gecco.server.actsofgod;

import gecco.server.automaton.*;
import gecco.server.clock.*;
import gecco.server.core.*;
import gecco.server.qmanager.*;
import gecco.util.*;

import java.util.*;

/**
 *  Description of the Class
 *
 * @author     Karl Hasselstrm
 * @created    June 27, 2001
 */
public class ActsOfGod extends SignalThread {
    /**
     *  Description of the Field
     */
    private TreeSet acts;
    /**
     *  Description of the Field
     */
    private Act next;


    /**
     *  Constructor for the ActsOfGod object
     *
     */
    public ActsOfGod() {
	super("Acts of God");
        acts = new TreeSet();
        next = null;
    }


    /**
     *  Adds a feature to the Act attribute of the ActsOfGod object
     *
     * @param  eventName       The feature to be added to the Act attribute
     * @param  factor          The feature to be added to the Act attribute
     * @param  x               The feature to be added to the Act attribute
     * @param  y               The feature to be added to the Act attribute
     * @param  time            The feature to be added to the Act attribute
     */
    public void addAct(String eventName, int factor, double x, double y,
            double time) {
        // Create the act.
        AutomatonEvent ae = new AutomatonEvent(eventName, factor);
        Act a = new Act(ae, x, y, time);

        // Add it to the list
        synchronized (acts) {
            acts.add(a);
        }

        // Interrupt the thread carrying out the acts in case the new
        // act should go first
        this.interrupt();
    }


    /**
     *  Main processing method for the ActsOfGod object
     */
    public void run() {

        // Loop forever
        while (true) {

	    // Check for signals.
	    switch (signalPending()) {
	    case SIGKILL:
	    case SIGQUIT:
		// Stop executing.
		return;
	    case SIGPAUSE:
		pauseHandler();
		clearSignal();
		break;
	    default:
		// Do nothing.
		clearSignal();
		break;
	    }

	    Clock clock = ReferenceHolder.getClock();

            // Get next act, or go to sleep if there is none.
            try {
                synchronized (acts) {
                    next = (Act) acts.first();
                }
            } catch (NoSuchElementException e) {

                // Sleep for a minute (real time) ...
                try {
                    clock.realSleep(60.0);
                } catch (InterruptedException e2) {
                    // This just means we were woken early. Go get
                    // next action!
                }

                // ... then go get next act!
                continue;
            }

            // Wait for the right time.
            double timeLeft = next.getTime() - clock.gameTimeNow();
            if (timeLeft > 0.0) {
                try {
                    clock.gameSleep(timeLeft);
                } catch (InterruptedException e) {
                    // Someone may have changed the set of acts. We
                    // need to start over!
                    continue;
                }
            }

            // Carry out the act and remove it from the todo list
            AutomatonEvent act = next.getAct();
            int x = (int) next.getX();
            int y = (int) next.getY();
            ReferenceHolder.getQManager().addEventToQueue(act, x, y);
	    synchronized (acts) {
		acts.remove(next);
	    }

            // Tell the world
            ReferenceHolder.log.println("Act of God '" + act.getEventName()
					+ "' effected.");
            ReferenceHolder.log.println("\tFactor = " + act.getFactor() +
					", position = (" + x + "," + y + ").");
        }
    }
}
