/***********************************************************************

GECCO, a platform for command and control gaming

Copyright (C) 2001 Henrik Brnhielm, Andreas Enblom, Niklas
Hallenfur, Karl Hasselstrm, Henrik Hgerstrm, Oskar Linde, Jing Fu
Zi and Jon slund

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or (at
your option) any later version.

This program is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
USA.

***********************************************************************/

package gecco.server.clock;

/**
 *  Description of the Class
 *
 * @author     Karl Hasselstrm
 * @created    July 17, 2001
 */
public class SleepingThread implements Comparable {

    /**
     *  Description of the Field
     */
    public Thread thread;

    /**
     *  Description of the Field
     */
    public double wakeupTime;


    /**
     *  Constructor for the SleepingThread object
     *
     * @param  thread      Description of Parameter
     * @param  wakeupTime  Description of Parameter
     */
    public SleepingThread(Thread thread, double wakeupTime) {
        this.thread = thread;
        this.wakeupTime = wakeupTime;
    }


    /**
     *  Tests whether two sleeping threads are equal.
     *
     * @param  other  Should be a <code>SleepingThread</code> .
     * @return        <code>true</code> if a is equal to this automaton, <code>
     *      false</code> otherwise.
     */
    public boolean equals(Object other) {
        return (compareTo(other) == 0);
    }


    /**
     *  Compares this sleeping thread with another sleeping thread. First, their
     *  wakeup times are compared, and if they are equal, the threads themselves
     *  are compared.
     *
     * @param  other  Should be a <code>SleepingThread</code> .
     * @return        -1 if <code>this</code> is less than <code>other</code> ,
     *      0 if they are equal, 1 if <code>this</code> is greater than <code>
     *      other</code> .
     */
    public int compareTo(Object other) {
        if (!(other instanceof SleepingThread)) {
            throw new ClassCastException("Cannot compare a SleepingThread with another object!");
        }
        SleepingThread st = (SleepingThread) other;
        if (this.wakeupTime < st.wakeupTime) {
            return -1;
        } else if (this.wakeupTime > st.wakeupTime) {
            return 1;
        }
        return this.thread.hashCode() - st.thread.hashCode();
    }


    public String toString() {
	return String.valueOf(thread) + " " + String.valueOf(wakeupTime);
    }

}
