/***********************************************************************

GECCO, a platform for command and control gaming

Copyright (C) 2001 Henrik Brnhielm, Andreas Enblom, Niklas
Hallenfur, Karl Hasselstrm, Henrik Hgerstrm, Oskar Linde, Jing Fu
Zi and Jon slund

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or (at
your option) any later version.

This program is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
USA.

***********************************************************************/

package gecco.server.clock;

import gecco.server.core.*;
import gecco.util.*;

/**
 *  A thread that periodically sends a new time string to the clients.
 *
 * @author     Karl Hasselstrm
 * @created July 17, 2001
 */
public class TimeUpdate implements SignalRunnable {

    /**
     *  Periodically sends the clients time update messages.
     */
    public void run() {
        // Loop forever.
        while (true) {

            // Sleep for between 1/20th and 1/10th second, real time.
            try {
                ReferenceHolder.getClock().realSleep((1.0 + Math.random()) / 20.0);
            } catch (InterruptedException e) {
                // Somebody woke us early, but that's ok, just carry on.
            }

            // Check for signals.
            switch (SignalThread.signalPending()) {
                case SIGKILL:
                case SIGQUIT:
                    // Stop executing.
                    return;
                case SIGPAUSE:
                    SignalThread.pauseHandler();
                    SignalThread.clearSignal();
                    break;
                default:
                    // Do nothing.
                    SignalThread.clearSignal();
                    break;
            }

            // Send time update message.
            String time = ReferenceHolder.getTimeString().getGameTime();
            ReferenceHolder.getVisibilityFilter().timeUpdate(time);
        }
    }
}
